package tools.descartes.dql.connector.kieker;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;

import kieker.common.configuration.Configuration;
import tools.descartes.dql.connector.AbstractOSGiQueryConnector;
import tools.descartes.dql.connector.PerformanceMetricsQueryConnector;
import tools.descartes.dql.connector.QueryProgressListener;
import tools.descartes.dql.connector.kieker.configgenerator.XMLGenerationManager;
import tools.descartes.dql.connector.kieker.filter.KiekerAnalysisController;
import tools.descartes.dql.connector.kieker.structures.Component;
import tools.descartes.dql.connector.kieker.structures.Resource;
import tools.descartes.dql.connector.kieker.structures.Trace;
import tools.descartes.dql.connector.kieker.structures.identifier.ResourceIdentifier;
import tools.descartes.dql.connector.kieker.structures.records.EventRecord;
import tools.descartes.dql.connector.kieker.structures.records.ResourceRecord;
import tools.descartes.dql.models.mapping.domain.ContinuousSeriesResult;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.SeriesResultElement;
import tools.descartes.dql.models.mapping.domain.StatType;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class PerformanceMetricsQueryConnectorImpl extends
		AbstractOSGiQueryConnector implements PerformanceMetricsQueryConnector {



	private final DomainFactory domainFactory = DomainFactory.eINSTANCE;


	@Override
	public EntityMapping requestMetrics(final EntityMapping request) {

		final EntityMapping response = EcoreUtil.copy(request);

		KiekerManager manager = new KiekerManager(request);
		manager.run();
		Trace result = manager.getAnalysisResult();
		long firstTs = manager.getStartingTime();
		

		// Process the trace back to a DQL result
		// Kieker ergebnisse filtern
		processPerformanceResults(response, result, firstTs);
		return response;
	}

	
	/* Modifies response to include the results of Kieker */
	private void processPerformanceResults(EntityMapping response, Trace kiekerResults, long firstTs) {
	
		Map<Resource, Trace> resources = kiekerResults.getResourceToTraceMap();
		Map<ResourceIdentifier, Resource> resourceIds = kiekerResults.getResourceIdentifierToResourceMap();
		Map<Component, Trace> services = kiekerResults.getComponentToTraceMap();

		// browse through each entity and its probes 
		//   and try to replace the probe with a result
		for (Entity ent : response.getEntities()) {
			for (Probe dqlProbe : ent.getProbes()) {		
				for (StatType st : dqlProbe.getStatTypes()) { 
					if (KiekerHelper.METRIC_UTILIZATION.equalsIgnoreCase(dqlProbe.getMetricName())) {
							if (st.getTypeName().equals("DEFAULT") || st.getTypeName().equals("SAMPLE")){
								ContinuousSeriesResult result = DomainFactory.eINSTANCE.createContinuousSeriesResult();
								result.setXAxisDescription("ms");
								result.setYAxisDescription("% (between 0 and 1)");
								ResourceIdentifier dqlId = ResourceIdentifier.createResourceIdentifier("", ent.getIdentifier());
								Resource res = resourceIds.get(dqlId);
								if (res != null  &&  resources.get(res)!= null && resources.get(res).getResources().size()> 0) {	
							
									try {
										for (ResourceRecord record : resources.get(res).getResources()) {
												//System.out.println("ADDING for "+ent.getIdentifier()+": Time" +record.getTimestamp()+" / Util: "+record.getContent().get(2));
												result.getElements().add(KiekerHelper.getSeriesResultElement(record.getTimestamp(), record.getContent().get(2), firstTs));
										}
										result.setValid(true);
										} catch (NumberFormatException exc) {
											result.setValid(false);
										}
										st.setResult(result);
								}
								
						}  else {
							// do nothing
						}
					} else if (KiekerHelper.METRIC_RESPONSE_TIME.equalsIgnoreCase(dqlProbe.getMetricName())) {
						if (st.getTypeName().equals("DEFAULT") || st.getTypeName().equals("SAMPLE")){
							ContinuousSeriesResult res = DomainFactory.eINSTANCE.createContinuousSeriesResult();
							res.setXAxisDescription("ms");
							res.setYAxisDescription("ms");
							if (services != null) {
							try {
									Component comp = new Component(KiekerHelper.getComponentString(ent.getIdentifier()));
									Trace tr = services.get(comp);
									if (tr != null && tr.getEvents().size()>0){
										for (EventRecord er : tr.getEvents()) {
											String dqlId = ent.getIdentifier();
											
											String kiekerId = er.getIdentifier().getComponentID();
											if (er.getIdentifier().isInterfaceIdentifier()){
												kiekerId += "." + er.getIdentifier().getInterfaceID();
											}
											if (dqlId.equals(kiekerId)) {
												res.getElements().add(KiekerHelper.getSeriesResultElement(er.getTimestamp(), er.getRuntime(), firstTs));
											}

										}
										res.setValid(true);
									}
									
								} catch (NumberFormatException exc) {
									res.setValid(false);
								}
								st.setResult(res);
							}
						}
						
					}
					
				}
			}
		}

	}
	
	
	

	

	

	public void reset() {
		// Nothing to do here
	}

	@Override
	public void addQueryProgressListener(QueryProgressListener arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public String getConstraint() {
		return null;
	}

	@Override
	public String getDoFConstraint() {
		return null;
	}

	@Override
	public String getDoFExplorationStrategy() {
		return null;
	}

	@Override
	public Properties getDoFExplorationStrategyProperties() {
		return null;
	}

	@Override
	public List<String> getSupportedConstraintNames() {
		return null;
	}

	@Override
	public List<String> getSupportedDoFConstraintNames() {
		return null;
	}

	@Override
	public List<String> getSupportedExplorationStrategyNames() {
		return null;
	}

	@Override
	public List<String> getSupportedTimeUnits() {
		return null;
	}

	@Override
	public List<EntityMapping> requestDoFMetrics(final EntityMapping request) {
		return null;
	}
	
	@Override
	public void setConstraint(final String constraint) {
	}

	@Override
	public void setDoFConstraint(final String doFConstraint) {
	}

	@Override
	public void setDoFExplorationStrategy(final String strategy,
			final Properties properties) {
	}

	@Override
	public void setObservationPeriod(final Date start, final Date end,
			final long sampleSize) {
	}

	@Override
	public void setObservationPeriod(final long amount, final String unit,
			final long sampleSize) {
	}

	@Override
	public void setObservationReference(final String type,
			final String identifier, final long sampleSize) {
	}

	@Override
	public boolean supportsConstraintRequests() {
		return false;
	}

	@Override
	public boolean supportsDoFRequests() {
		return false;
	}

	@Override
	public boolean supportsMetricRequests() {
		return true;
	}

	@Override
	public boolean supportsObservationPeriods() {
		return false;
	}
}