package tools.descartes.dql.connector.kieker.filter;

import java.io.File;

import tools.descartes.dql.connector.kieker.structures.Trace;
import tools.descartes.dql.connector.kieker.structures.classes.ClassMap;
import tools.descartes.dql.connector.kieker.structures.cpu.CpuMap;
import tools.descartes.dql.connector.kieker.structures.interfaces.InterfaceMap;
import tools.descartes.dql.connector.kieker.structures.resources.ResourceMap;

public class KiekerTraceLoader {
	private Trace trace;

	public KiekerTraceLoader(Trace trace) {
		this.trace=trace;
	}

	public KiekerTraceLoader() {
	}

	public Trace loadKiekerTraceFromFile(File kiekerTraceFile) {
		if (!(kiekerTraceFile == null || !kiekerTraceFile.canRead())) {
			System.out.println("invalid filepath given, system will shutdown");
			System.exit(1);
		}
		String[] dir = new String[] { kiekerTraceFile.getAbsolutePath() };
		return loadKiekerTraceFromDirectories(dir);
	}

	public Trace loadKiekerTraceFromDirectories(String[] kiekerInputDirs) {
		clearMaps();
		KiekerReader kiekerReader = new KiekerReader(kiekerInputDirs);
		kiekerReader.read();
		return fetchInformationFromMaps();
	}
	
	private void clearMaps(){
		ClassMap.getInstance().clear();
		InterfaceMap.getInstance().clear();
		ResourceMap.getInstance().clear();
		CpuMap.getInstance().clear();
	}

	private Trace fetchInformationFromMaps() {
		if (trace == null) {
			trace = new Trace();
		}
		ClassMap.getInstance().pushToTrace(trace);
		InterfaceMap.getInstance().pushToTrace(trace);
		ResourceMap.getInstance().pushToTrace(trace);
		CpuMap.getInstance().pushToTrace(trace);
		// MemSwapMap.getInstance().pushToTrace(trace);
		// more information can be included from other Maps
		return trace;
	}
	
	
}
